-----------------------------------------------------------------------------------------------
-- Client Lua Script for SoundCharge
-----------------------------------------------------------------------------------------------
require "Window"
--------------------------------------------------------------------------
-- SoundCharge Module Definition
-----------------------------------------------------------------------------------------------
local SoundCharge = {} 
local GeminiLocale, L
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
local nbCharge = 0
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function SoundCharge:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 
	o.bSoundChargeIsOn = true
	o.nSoundChargeOptions = {
		{
				chargelevel = 1,
				soundlabel = "Sound 1",
				soundId = 176
		},
		{
				chargelevel = 2,
				soundlabel = "Sound 2",
				soundId = 177
		},
		{
				chargelevel = 3,
				soundlabel = "Sound 3",
				soundId = 178
		},
		{
				chargelevel = 4,
				soundlabel = "Sound 4",
				soundId = 179
		}
	}
    return o
end
function SoundCharge:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {"Gemini:Locale-1.0"}
	--If not esper or spellslinger do not load
	local unitPlayer = GameLib.GetPlayerUnit()
	if unitPlayer then
		if unitPlayer:GetClassId() ~= GameLib.CodeEnumClass.Esper and unitPlayer:GetClassId() ~= GameLib.CodeEnumClass.Spellslinger then
			return 	
		end	
	end	
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
-----------------------------------------------------------------------------------------------
-- SoundCharge OnLoad
-----------------------------------------------------------------------------------------------
function SoundCharge:OnLoad()
	self.xmlDoc = XmlDoc.CreateFromFile("SoundCharge.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
	GeminiLocale = Apollo.GetPackage("Gemini:Locale-1.0").tPackage
    L = GeminiLocale:GetLocale("SoundCharge", false)
	Apollo.LoadSprites("SoundCharge_img.xml")
end
-----------------------------------------------------------------------------------------------
-- SoundCharge OnDocLoaded
-----------------------------------------------------------------------------------------------
function SoundCharge:OnDocLoaded()
	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "SoundChargeForm", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
	    self.wndMain:Show(false, true)
		self.wndMain:FindChild("GeneralBtn"):SetCheck(true)
		if self.bSoundChargeIsOn then
			self.wndMain:FindChild("MuteSwitch"):SetCheck(false)
		else
			self.wndMain:FindChild("MuteSwitch"):SetCheck(true)
		end
		self.xmlDoc = nil
		
		--
		Apollo.RegisterSlashCommand("schargeoff", "OnSoundChargeOff", self)
		Apollo.RegisterSlashCommand("schargeon", "OnSoundChargeOn", self)
		Apollo.RegisterSlashCommand("schargehelp", "OnSoundChargeHelp", self)
		Apollo.RegisterSlashCommand("schargeconfig", "OnSoundChargeConfig", self)
		Apollo.RegisterEventHandler("InterfaceMenuListHasLoaded", "OnInterfaceMenuListHasLoaded", self)
		Apollo.RegisterEventHandler("ShowSCMenu", "OnShowSCMenu", self)
		Apollo.RegisterEventHandler("OptionsUpdated_FFPreferences","InitializeControls", self)
		Apollo.RegisterEventHandler("StartSpellThreshold", 	"OnStartSpellThreshold", self)
		Apollo.RegisterEventHandler("ClearSpellThreshold", 	"OnClearSpellThreshold", self)
		Apollo.RegisterEventHandler("UpdateSpellThreshold", "OnUpdateSpellThreshold", self)
		--
		self.mapDDParents = {
			--hud options
			{
				wnd = self.wndMain:FindChild("DropToggleChargeOneSound"),
				nConsoleVar = "general.ChargeOne",
				strRadioGroup = "GeneralChargeOneGroup"
			},
			{
				wnd = self.wndMain:FindChild("DropToggleChargeTwoSound"),
				nConsoleVar = "general.ChargeTwo",
				strRadioGroup = "GeneralChargeTwoGroup"
			},
			{
				wnd = self.wndMain:FindChild("DropToggleChargeThreeSound"),
				nConsoleVar = "general.ChargeThree",
				strRadioGroup = "GeneralChargeThreeGroup"
			},
			{
				wnd = self.wndMain:FindChild("DropToggleChargeFourSound"),
				nConsoleVar = "general.ChargeFour",
				strRadioGroup = "GeneralChargeFourGroup"
			}
		}
		for idx, wndDD in pairs(self.mapDDParents) do
			wndDD.wnd:AttachWindow(wndDD.wnd:FindChild("ChoiceContainer"))
			wndDD.wnd:FindChild("ChoiceContainer"):Show(false)
		end		
		-- Trads
		GeminiLocale:TranslateWindow(L, self.wndMain)
		self.wndMain:FindChild("Notice"):SetText(L["helpnotice"])
		self.wndMain:FindChild("HelpPanel"):SetText(L["helptext"])
		self:InitializeControls()
	end
end
-----------------------------------------------------------------------------------------------
-- SoundCharge OnSave
-----------------------------------------------------------------------------------------------
function SoundCharge:OnSave(eType)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Character then
		return
	end
	local tSave =
	{
		bSoundChargeIsOn  = self.bSoundChargeIsOn,
		nSoundChargeOptions = self.nSoundChargeOptions,
	}
	return tSave
end
-----------------------------------------------------------------------------------------------
-- SoundCharge OnRestore
-----------------------------------------------------------------------------------------------
function SoundCharge:OnRestore(eType, tSavedData)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Character then
		return
	end
	
	if tSavedData then
		if tSavedData.bSoundChargeIsOn then
			self.bSoundChargeIsOn  = true
		else
			self.bSoundChargeIsOn  = false
		end	
		
		if tSavedData.nSoundChargeOptions then
			self.nSoundChargeOptions= tSavedData.nSoundChargeOptions
		end
	end
end
-----------------------------------------------------------------------------------------------
-- SoundCharge Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here
function SoundCharge:InitializeControls()
	for idx, option in pairs(self.nSoundChargeOptions) do
		if option.soundlabel then
			self.mapDDParents[idx].wnd:SetText(option.soundlabel); 
		end	
	end
end

function SoundCharge:OnInterfaceMenuListHasLoaded()
	Event_FireGenericEvent("InterfaceMenuList_NewAddOn", "SoundCharge", {"ShowSCMenu", "", ""})
end
---------------------------------------------------------------------------------------------------
-- SoundChargeForm Functions
---------------------------------------------------------------------------------------------------
function SoundCharge:OnShowSCMenu()
	self.wndMain:FindChild("HelpPanel"):SetText(L["helptext"])
	self.wndMain:Invoke()
end
function SoundCharge:OnCloseWindow()
	self.wndMain:Close()
end
function SoundCharge:OnGeneralOptionsCheck(wndHandler, wndControl)
	self.wndMain:FindChild("Content:General"):Show(true)
	self.wndMain:FindChild("Content:Help"):Show(false)
end
function SoundCharge:OnHelpOptionsCheck( wndHandler, wndControl, eMouseButton )
	self.wndMain:FindChild("Content:General"):Show(false)
	self.wndMain:FindChild("Content:Help"):Show(true)
	self.wndMain:FindChild("HelpPanel"):SetText(L["helptext"])
end
function SoundCharge:OnHUDRadio( wndHandler, wndControl, eMouseButton )
	for idx, wndDD in pairs(self.mapDDParents) do
		if wndDD.wnd == wndControl:GetParent():GetParent():GetParent() then
			-- INIT SOUNDS ID HERE
			self.nSoundChargeOptions[idx].soundlabel=wndControl:GetText(); 
			if wndControl:GetName() == "-1" then
				self.nSoundChargeOptions[idx].soundId = -1;
			elseif wndControl:GetName() == "0" then
				Sound.PlayFile(idx..".wav")
				self.nSoundChargeOptions[idx].soundId = 0
			elseif wndControl:GetName() then
				Sound.Play(wndControl:GetName())
				self.nSoundChargeOptions[idx].soundId = tonumber(wndControl:GetName())
			end
			wndControl:GetParent():GetParent():SetText(wndControl:GetText())
			break
		end
	end

	Event_FireGenericEvent("OptionsUpdated_FFPreferences")
	wndControl:GetParent():GetParent():Show(false)
end
function SoundCharge:OnMute()
	if self.bSoundChargeIsOn then
			self.bSoundChargeIsOn = false
			ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_System, "SoundCharge is now off")
	else
			self.bSoundChargeIsOn = true
			ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_System, "SoundCharge is now on")	
	end
end
function SoundCharge:OnSoundChargeOff()
	self.bSoundChargeIsOn = false
	self.wndMain:FindChild("MuteSwitch"):SetCheck(true)
	ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_System, "SoundCharge is now off")
end
function SoundCharge:OnSoundChargeOn()
	self.bSoundChargeIsOn = true
	self.wndMain:FindChild("MuteSwitch"):SetCheck(false)
	ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_System, "SoundCharge is now on")
end
function SoundCharge:OnSoundChargeHelp()
	ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_System, "SoundCharge commands :")
	ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_System, "- /ffon enable sound alert")
	ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_System, "- /ffoff disable sound alert")
	ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_System, "- /ffconfig sound configuration")
end
function SoundCharge:OnSoundChargeConfig()
	self.wndMain:FindChild("Content:General"):Show(true)
	self.wndMain:FindChild("Content:Help"):Show(false)
	self.wndMain:Invoke()
end
-----------------------------------------------------------------------------------------------
-- SoundCharge CORE Function
-----------------------------------------------------------------------------------------------
function SoundCharge:OnStartSpellThreshold(spellId, nb, eCastMethod)
	if eCastMethod and eCastMethod == 7 and self.bSoundChargeIsOn then
		if nbCharge == 0 then
			if self.nSoundChargeOptions[1].soundId > 0 then
				Sound.Play(self.nSoundChargeOptions[1].soundId)
			elseif self.nSoundChargeOptions[1].soundId == 0 then
				Sound.PlayFile("1.wav");
			end
			nbCharge = nbCharge +1
		end
	end
end
function SoundCharge:OnUpdateSpellThreshold(spellId, nb)
	if self.bSoundChargeIsOn then
		if nbCharge < 5 then
			nbCharge = nbCharge +1
			if self.nSoundChargeOptions[nbCharge].soundId > 0 then
				Sound.Play(self.nSoundChargeOptions[nbCharge].soundId)
			elseif self.nSoundChargeOptions[nbCharge].soundId == 0 then
				Sound.PlayFile(nbCharge..".wav");
			end
		end
	end
end
function SoundCharge:OnClearSpellThreshold(spellId)
	nbCharge =0
end
-----------------------------------------------------------------------------------------------
-- SoundCharge Instance
-----------------------------------------------------------------------------------------------
local SoundChargeInst = SoundCharge:new()
SoundChargeInst:Init()